/**** Regression Discontinuity Estimator with Optimal Bandwidth ****/



version 10.0



capture program drop rdob_adj

program define rdob_adj

syntax anything [if] [in] [,uniform detail c(real 0) fuzzy(varname)]

tokenize "`anything'"

local y `1'

local x `2'



/*Following are the temporary variables this program will use throughout*/ 



tempvar Y1 Y2 temp1 temp2 temp3 temp4 cons T1 T2 temp5 ind temp6 lambda xindl xindr xcl xcr xc

tempvar el er elx erx e







/*** Calculation of the Optimal Bandwidth ***/



/**Step 1: Estimation of density f(c) and conditional variance sigma-squared(c)**/



/*Calculating the pilot bandwidth h1*/



quietly summarize `x', detail

local Sx = r(sd)

local N = r(N)

local h1 = 1.84*`Sx'*(`N'^(-1/5))



/*Calculating the number of units on either side of the threshold

and the average outcomes*/



quietly count if (`c'-`h1')<=`x' & `x'<`c'

local Nh1n = r(N)

quietly count if `c'<=`x' & `x'<=(`c'+`h1')

local Nh1p = r(N)



quietly ameans `y' if (`c'-`h1')<=`x' & `x'<`c'

local Yh1n = r(mean)

quietly ameans `y' if `c'<=`x' & `x'<=(`c'+`h1')

local Yh1p = r(mean)



/*Estimating the density of X at c and the conditional variance of Y

given X_i=c at x=c */



local fxc = (`Nh1p'+`Nh1n')/(2*`N'*`h1')



generate `Y1' =(`y'-`Yh1n')^2 

quietly summarize `Y1' if (`c'-`h1')<=`x' & `x'<`c', detail

local Y1sum =r(sum)

generate `Y2' =(`y'-`Yh1p')^2 

quietly summarize `Y2' if `c'<=`x' & `x'<=(`c'+`h1'), detail

local Y2sum =r(sum)



local sigma2c = (1/(`Nh1p'+`Nh1n'))*(`Y1sum'+`Y2sum')





/**Step 2: Estimation of second derivatives**/



/*Calculating the pilot bandwidth h2*/



quietly summarize `x' if `x'>=`c', detail

scalar medXp = r(p50)

quietly summarize `x' if `x'<`c', detail

scalar medXn = r(p50)



generate `temp1'=(`x'>=`c')

generate `temp2'=`x'-`c'

generate `temp3'=`temp2'^2

generate `temp4'=`temp2'^3



quietly regress `y' `temp1' `temp2' `temp3' `temp4' if `x' >= medXn & `x' <= medXp



local m3c=6*_b[`temp4'] 



quietly count if `x'>=`c'

local Np=r(N)

quietly count if `x'<`c'

local Nn=r(N)



local h2p=3.56*((`sigma2c'/(`fxc'*max((`m3c')^2, 0.01)))^(1/7))*(`Np'^(-1/7))

local h2n=3.56*((`sigma2c'/(`fxc'*max((`m3c')^2, 0.01)))^(1/7))*(`Nn'^(-1/7))



/*Calculating the estimates of the second derivatives*/



quietly count if `c'<=`x' & `x'<=(`c'+`h2p')

local N2p=r(N)

quietly count if (`c'-`h2n')<=`x' & `x'<`c'

local N2n=r(N)



generate `cons'=1

generate `T1'=`x'-`c'

generate `T2'=`T1'^2



quietly regress `y' `cons' `T1' `T2' if `c'<=`x' & `x'<=(`c'+`h2p')

local m2pc=2*_b[`T2']

quietly regress `y' `cons' `T1' `T2' if (`c'-`h2n')<=`x' & `x'<`c'

local m2nc=2*_b[`T2'] 





/**Step 3: Calculation of regularization terms and calculation of the optimal bandwidth**/



/*Calculating the regularization terms*/



local rp=(720*`sigma2c')/(`N2p'*((`h2p')^4))

local rn=(720*`sigma2c')/(`N2n'*((`h2n')^4))



/*Calculating the optimal bandwidth*/



if "`uniform'"~="" {

local CK = 5.4

}

else {

local CK = 3.4375

}



global bw= `CK'*(((2*`sigma2c')/(`fxc'*(((`m2pc'-`m2nc')^2)+(`rp'+`rn'))))^(1/5))*`N'^(-1/5)




end





/* end rdob_adj.ado */
